@@ -1,5 +1,6 @@
 from .text_utils import *
 from .audio_utils import *
+from urllib.parse import quote
 from .functions import (logger,
                         create_key_value,
                         os,
@@ -10,49 +11,131 @@ from .functions import (logger,
                         url_join)
 from pydub import AudioSegment
 from moviepy.editor import VideoFileClip
-def generate_info_json(filename: str, prompt: str):
+from datetime import datetime
+
+EXT_TO_PREFIX = {
+    **dict.fromkeys(
+        {'.jpg', '.jpeg', '.png', '.gif', '.bmp', '.tiff', '.webp', '.svg'},
+        'infos'
+    ),
+    **dict.fromkeys(
+        {'.mp4', '.avi', '.mov', '.wmv', '.flv', '.mkv', '.webm'},
+        'videos'
+    ),
+    '.pdf': 'pdfs',
+    **dict.fromkeys({'.mp3', '.wav', '.ogg', '.flac', '.aac', '.m4a'}, 'audios'),
+    **dict.fromkeys({'.doc', '.docx', '.txt', '.rtf'}, 'docs'),
+    **dict.fromkeys({'.ppt', '.pptx'}, 'slides'),
+    **dict.fromkeys({'.xls', '.xlsx', '.csv'}, 'sheets'),
+    **dict.fromkeys({'.srt'}, 'srts'),
+}
+def generate_with_bigbird(text: str, task: str = "title", model_dir: str = "allenai/led-base-16384") -> str:
+    try:
+        tokenizer = LEDTokenizer.from_pretrained(model_dir)
+        model = LEDForConditionalGeneration.from_pretrained(model_dir)
+        prompt = (
+            f"Generate a concise, SEO-optimized {task} for the following content: {text[:1000]}"
+            if task in ["title", "caption", "description"]
+            else f"Summarize the following content into a 100-150 word SEO-optimized abstract: {text[:4000]}"
+        )
+        inputs = tokenizer(prompt, return_tensors="pt", max_length=512, truncation=True)
+        outputs = model.generate(
+            inputs["input_ids"],
+            max_length=200 if task in ["title", "caption"] else 300,
+            num_beams=5,
+            early_stopping=True
+        )
+        return tokenizer.decode(outputs[0], skip_special_tokens=True)
+    except Exception as e:
+        print(f"Error in BigBird processing: {e}")
+        return ""
+def generate_media_url(fs_path: str,domain=None,repository_dir=None) -> str | None:
+    fs_path = os.path.abspath(fs_path)
+    if not fs_path.startswith(repository_dir):
+        return None
+    rel_path = fs_path[len(repository_dir):]
+    rel_path = quote(rel_path.replace(os.sep, '/'))
+    ext = os.path.splitext(fs_path)[1].lower()
+    prefix = EXT_TO_PREFIX.get(ext, 'repository')
+    return f"{domain}/{prefix}/{rel_path}"
+def get_image_metadata(file_path):
+    """Extract image metadata (dimensions, file size)."""
+    try:
+        with Image.open(file_path) as img:
+            width, height = img.size
+            file_size = get_file_size(file_path)
+        return {
+            "dimensions": {"width": width, "height": height},
+            "file_size": round(file_size, 3)
+        }
+    except Exception as e:
+        return {"dimensions": {"width": 0, "height": 0}, "file_size": 0}
+def generate_info_json(filepath=None,
+                        prompt=None,
+                        alt_text=None,
+                        title=None,
+                        description=None,
+                        keywords=None,
+                        domain=None,
+                        video_path=None,
+                        repository_dir=None,
+                        generator=None,
+                        LEDTokenizer=None,
+                        LEDForConditionalGeneration=None):
     """
     Build structured info.json for an image, including SEO schema and social metadata.
     """
-    file_stem = Path(filename).stem
-    ext = Path(filename).suffix.upper().lstrip('.')
-    file_path = Path(IMGS_DIR) / filename
-    url_path = f"/imgs/{filename}"
-
-    img_meta = get_image_metadata(str(file_path)) if file_path.exists() else {"dimensions": {"width": 0, "height": 0}, "file_size": 0.0}
-    gen = generator(prompt, max_length=100, num_return_sequences=1)[0]
-    description = gen.get('generated_text', '')[:150]
-
+    dirname = os.path.dirname(filepath)
+    basename = os.path.basename(filepath)
+    filename,ext = os.path.splitext(basename)
+    url_path = filepath
+    title_prompt = generate_with_bigbird(f"Video of {filename.replace('-', ' ')} with the video text f{alt_text}", task="title")
+    description_prompt = generate_with_bigbird(f"Video of {filename.replace('-', ' ')} with the video text f{alt_text}", task="description")
+    caption_prompt = generate_with_bigbird(f"Video of {filename.replace('-', ' ')} with the video text f{alt_text}", task="caption")
+    img_meta = get_image_metadata(str(filepath)) if os.path.isfile(filepath) else {"dimensions": {"width": 0, "height": 0}, "file_size": 0.0}
+    dimensions = img_meta.get("dimensions",{})
+    width = dimensions.get('width')
+    height = dimensions.get('height')
+    file_size = img_meta.get("file_size")
+    description = alt_text
+    title = filename
+    caption = alt_text
+    if generator:
+        gen = generator(prompt, max_length=100, num_return_sequences=1)[0]
+        description = gen.get('generated_text', '')[:150]
+    description = alt_text
+    title = title or filename
+    caption = caption or alt_text
     info = {
-        "alt": file_stem,
-        "caption": file_stem,
-        "keywords_str": file_stem,
-        "filename": file_stem,
-        "ext": f".{ext}",
-        "title": f"{filename} ({img_meta['dimensions']['width']}×{img_meta['dimensions']['height']})",
-        "dimensions": img_meta["dimensions"],
-        "file_size": img_meta["file_size"],
+        "alt": alt_text,
+        "caption": alt_text,
+        "keywords_str": keywords,
+        "filename": filename,
+        "ext": f"{ext}",
+        "title": f"{title} ({width}×{height})",
+        "dimensions": dimensions,
+        "file_size": file_size,
         "license": "CC BY-SA 4.0",
         "attribution": "Created by thedailydialectics for educational purposes",
-        "longdesc": None,
+        "longdesc": description,
         "schema": {
             "@context": "https://schema.org",
             "@type": "ImageObject",
-            "name": file_stem,
+            "name": filename,
             "description": description,
-            "url": f"{DOMAIN}{url_path}",
-            "contentUrl": f"{DOMAIN}{url_path}",
-            "width": img_meta['dimensions']['width'],
-            "height": img_meta['dimensions']['height'],
+            "url": generate_media_url(filepath,domain=domain,repository_dir=repository_dir),
+            "contentUrl": generate_media_url(video_path,domain=domain,repository_dir=repository_dir),
+            "width": width,
+            "height": height,
             "license": "https://creativecommons.org/licenses/by-sa/4.0/",
             "creator": {"@type": "Organization", "name": "thedailydialectics"},
             "datePublished": datetime.now().strftime("%Y-%m-%d")
         },
         "social_meta": {
-            "og:image": f"{DOMAIN}{url_path}",
-            "og:image:alt": None,
+            "og:image": generate_media_url(filepath,domain=domain,repository_dir=repository_dir),
+            "og:image:alt": alt_text,
             "twitter:card": "summary_large_image",
-            "twitter:image": f"{DOMAIN}{url_path}"
+            "twitter:image": generate_media_url(filepath,domain=domain,repository_dir=repository_dir)
         }
     }
     return info
@@ -136,11 +219,19 @@ def pick_optimal_thumbnail(whisper_result, keywords, thumbnails_directory,*args,
     scores = []
     dirbase = os.path.basename(os.path.dirname(thumbnails_directory))
     
-    thumb_name,thumb_ext = os.path.splitext(os.listdir(thumbnails_directory)[0]) 
+    thumb_name,thumb_ext = os.path.splitext(os.listdir(thumbnails_directory)[0])
+    thubnail_items = os.listdir(thumbnails_directory)
+    thubnail_items_length = len(thubnail_items)
+    if thubnail_items_length> 35:
+        thubnail_items = thubnail_items[35:]
+    thubnail_items_length = len(thubnail_items)
+    if thubnail_items_length> 35:
+        thubnail_items = thubnail_items[:-35]
+    
     # Get list of thumbnails from directory
     thumbnails = sorted(
-        [f for f in os.listdir(thumbnails_directory) if f.endswith(".jpg")],
-        key=lambda x: get_frame_number(x)
+        [f for f in thubnail_items if f.endswith(".jpg")],
+        key=lambda x: get_frame_number(x) 
     )
     
     # Process each Whisper segment
@@ -180,8 +271,16 @@ def get_seo_data(info,
                  uploader=None,
                  domain=None,
                  categories=None,
-                 videos_url=None):
-    
+                 videos_url=None,
+                 repository_dir=None,
+                 directory_links=None,
+                 videos_dir=None,
+                 infos_dir=None,
+                 base_url=None,
+                 generator=None,
+                 LEDTokenizer=None,
+                 LEDForConditionalGeneration=None):
+
     info = create_key_value(info,
                             'categories',
                             categories or {'ai': 'Technology', 'cannabis': 'Health', 'elon musk': 'Business'})
@@ -197,49 +296,78 @@ def get_seo_data(info,
     info = create_key_value(info,
                             'videos_url',
                             videos_url or f"{info['domain']}/videos")
-    
+    video_path = info.get('video_path')
     for keyword_key in ['combined_keywords','keywords']:
         keywords = info.get(keyword_key,[])
         if keywords and len(keywords)>0:
             break
-    primary_keyword = keywords[0] if keywords and len(keywords)>0 else info['filename']
-    seo_title = f"{primary_keyword} - {info['filename']}"
-    info['seo_title'] = get_from_list(seo_title,length=70)
+    filename = info.get('filename')
+    if not filename:
+        basename = os.path.basename(video_path)
+        filename,ext = os.path.splitext(basename)
+        info['basename'] = basename
+        info['filename'] = filename
+    info['title'] = info.get('title',filename)
+    primary_keyword = keywords[0] if keywords and len(keywords)>0 else filename
+    seo_title = f"{primary_keyword} - {info['title']}"
+    seo_title = get_from_list(seo_title,length=70)
+    info['seo_title'] = seo_title
+    
     summary = info.get('summary','')
     summary_desc = get_from_list(summary,length=150)
     keywords_str = ', '.join(get_from_list(keywords,length=3))
     seo_desc = f"{summary_desc} Explore {keywords_str}. Visit thedailydialectics.com for more!"
-    info['seo_description'] = get_from_list(seo_desc,length=300)
+    seo_description = get_from_list(seo_desc,length=300)
+    info['seo_description'] = seo_description
+    
+    seo_tags = [kw for kw in keywords if kw.lower() not in ['video','audio','file']]
+    info['seo_tags'] = seo_tags
     
-    info['seo_tags'] = [kw for kw in keywords if kw.lower() not in ['video','audio','file']]
     video_text = info.get('video_text')
     info['thumbnail']={}
     if video_text and len(video_text)>0:
         thumnail_data = video_text[0]
-        info['thumbnail']['file_path']= os.path.join(info['thumbnails_directory'],
-                                                     thumnail_data.get("frame"))
+        thumbnail_filepath = os.path.join(info['thumbnails_directory'],thumnail_data.get("frame"))
+        info['thumbnail']['file_path'] = thumbnail_filepath
+        
+        thumbnail_alt_text = thumnail_data.get("text")
+        info['thumbnail']['alt_text']= thumbnail_alt_text
         
-        info['thumbnail']['alt_text']= thumnail_data.get("text")
     whisper_json = info["whisper_result"]
-    thumbnail_score = pick_optimal_thumbnail(whisper_json,
-                                               keywords,
-                                              info["thumbnails_directory"])
+    thumbnail_score = pick_optimal_thumbnail(whisper_json,keywords,info["thumbnails_directory"])
     if thumbnail_score:
         best_frame, score, matched_text = thumbnail_score
+        file_path = os.path.join(info['thumbnails_directory'],best_frame)
+        info['thumbnail']['file_path']= file_path
         
-        info['thumbnail']['file_path']= os.path.join(info['thumbnails_directory'],
-                                                     best_frame)
+        alt_text = get_from_list(matched_text,length=100)
+        info['thumbnail']['alt_text']= alt_text
         
-        info['thumbnail']['alt_text']= get_from_list(matched_text,length=100)
+        basename = os.path.basename(file_path)
+        filename,ext = os.path.splitext(basename)
         
-        prompt = f"Generate SEO metadata for {filename} with the video text info['thumbnail']['alt_text']"
-        info['thumbnail']['seo_data']= generate_info_json(info['thumbnail']['file_path'], prompt)
+        prompt = f"Generate SEO metadata for {filename} with the video text f{alt_text}"
+        thumbnail_seo_data = generate_info_json(file_path,
+                                                prompt,
+                                                alt_text,
+                                                seo_title,
+                                                seo_description,
+                                                keywords,
+                                                domain,
+                                                video_path,
+                                                repository_dir,
+                                                generator,
+                                                LEDTokenizer,
+                                                LEDForConditionalGeneration)
+        info['thumbnail']['seo_data']= thumbnail_seo_data
     
     audio = AudioSegment.from_wav(info['audio_path'])
-    
-    info['duration_seconds'] = len(audio) / 1000
-    
-    info['duration_formatted'] = format_timestamp(len(audio))
+
+    duration_seconds = len(audio) / 1000
+    info['duration_seconds'] = duration_seconds
+
+    duration_formatted = format_timestamp(len(audio))
+    info['duration_formatted'] = duration_formatted
     
     
 
